package com.supervision.pdfqaserver;

import cn.hutool.core.date.TimeInterval;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.supervision.pdfqaserver.constant.DocumentContentTypeEnum;
import com.supervision.pdfqaserver.domain.PdfAnalysisOutput;
import com.supervision.pdfqaserver.domain.TextVector;
import com.supervision.pdfqaserver.dto.*;
import com.supervision.pdfqaserver.service.*;
import lombok.extern.slf4j.Slf4j;
import org.junit.jupiter.api.Test;
import org.neo4j.driver.*;
import org.neo4j.driver.Record;
import org.springframework.ai.embedding.Embedding;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import java.util.*;
import java.util.stream.Collectors;
import static org.neo4j.driver.Values.parameters;

@Slf4j
@SpringBootTest
class PdfQaServerApplicationTests {

    @Autowired
    private KnowledgeGraphService knowledgeGraphService;
    @Test
    void generateGraphTest() {
        knowledgeGraphService.generateGraph("15");
        log.info("finish...");
    }

    @Test
    void testGenerateGraph2() {
        List<EREDTO> eredtos = knowledgeGraphService.listPdfEREDTO("15");

        knowledgeGraphService.generateGraphSimple(eredtos);
        log.info("finish...");
    }

    @Autowired
    private Driver driver;

    /**
     * 测试保存或更新节点
     */
    @Test
    void testSaveOrUpdateEntityNode() {

        List<Long> longs = saveOrUpdateRelation(7838L, 7940L, "REL_TYPE", true, false, Map.of("name", "test1"));
        System.out.println(longs);
    }
    public void saveOrUpdateEntityNode(String label, String uniqueKey, Map<String, Object> properties) {

        try (Session session = driver.session()) {
            // MERGE语句确保唯一性
            String query = String.format(
                    "MERGE (n:%s {%s: $uniqueValue}) " +
                            "SET n += $properties " +
                            "RETURN id(n)  as id",
                    label, uniqueKey);

            Result result = session.run(query, parameters("uniqueValue", properties.get(uniqueKey), "properties", properties));


            if (result.hasNext()) {
                Record next = result.next();
                Value value = next.get("id");
                long aLong = value.asLong();
                System.out.printf("已处理 %s 节点: %s%n", label, next.keys());
            }
        }
    }

    private List<Long> saveOrUpdateRelation(Long sourceId, Long targetId, String relationType,
                                            boolean leftDirection, boolean rightDirection, Map<String, Object> relation) {

        try (Session session = driver.session()) {
            // MERGE关系
            String query = "MATCH (a) WHERE id(a) = $sourceId " +
                    "MATCH (b) WHERE id(b) = $targetId " +
                    "MERGE (a)%s-[r:" + relationType + "]-%s(b) " +
                    "SET r += $properties " +
                    "RETURN id(r) as id";

            String format = String.format(query,
                    leftDirection ? "<" : "",
                    rightDirection ? ">" : "");

            Result result = session.run(format,
                    parameters("sourceId", sourceId, "targetId", targetId, "properties", relation));

            return result.stream().map(record -> record.get("id").asLong()).collect(Collectors.toList());
        }
    }
    @Autowired
    private  ChinesEsToEnglishGenerator chinesEsToEnglishGenerator;
    @Test
    void testChinesEsToEnglishGenerator() {
        String chinese = "你好";
        String english = chinesEsToEnglishGenerator.generate(chinese);
        System.out.println("翻译结果: " + english);
    }

    @Autowired
    TripleConversionPipeline tripleConversionPipeline;
    @Test
    void testChinesEsToEnglishGenerator2() {
        String s = """
                 |                                          | 项<br>日  |  |      | 附注 | 2023年12月31日        | 2023年1月1日          |
                 |------------------------------------------|---------|--|------|----|--------------------|--------------------|
                 | 流动资产:                                    |         |  |      |    |                    |                    |
                 | 货币资金                                     |         |  | 六、1  |    | 4,879,272,436.13   | 20,493,232,077.05  |
                """;
        Boolean classify = tripleConversionPipeline.classify(s);
        System.out.println(classify);
    }

    @Test
    void makeOutPdfContentTypeTest() {
        DocumentContentTypeEnum documentContentTypeEnum = tripleConversionPipeline.makeOutPdfContentType(5);
        System.out.println(documentContentTypeEnum);
    }

    @Test
    void makeOutPdfIndustryTest() {
        String industry = tripleConversionPipeline.makeOutPdfIndustry(5);
        System.out.println(industry);
    }

    @Test
    void makeOutTruncationIntentTest() {
        TruncateDTO truncateDTO = new TruncateDTO();
        truncateDTO.setContent("# 2、同时按照境外会计准则与按照中国会计准则披露的财务报告中净利润和净资产差异情况 \n" +
                "\n" +
                "□适用 回不适用 ");
        List<String> strings = tripleConversionPipeline.makeOutTruncationIntent(truncateDTO);
        System.out.println(strings);
    }

    @Test
    void makeOutTruncationIntentTest2() {
        TruncateDTO truncateDTO = new TruncateDTO();
        truncateDTO.setContent("# 2、同时按照境外会计准则与按照中国会计准则披露的财务报告中净利润和净资产差异情况 \n" +
                "\n" +
                "□适用 回不适用 ");
        IntentDTO intentDTO = new IntentDTO();
        intentDTO.setDigest("财务报告差异分析");
        List<IntentDTO> strings = tripleConversionPipeline.makeOutTruncationIntent(truncateDTO,List.of(intentDTO));
        System.out.println(strings);
    }

    @Test
    public void metaDataTrainTest() {

        knowledgeGraphService.metaDataTrain(15);
    }

    @Test
    void generateGraphBaseTrainTest() {

        knowledgeGraphService.generateGraphBaseTrain(15);
    }

    @Autowired
    private AiCallService aiCallService;
    @Test
    void aiCallServiceCallTest() {

        String call = aiCallService.call("你好");
        System.out.println(call);
    }

    @Test
    void resetGraphDataTest() {
        knowledgeGraphService.resetGraphData("15");
    }

    @Autowired
    private PdfAnalysisOutputService pdfAnalysisOutputService;
    @Test
    void queryGraphTest() {
        List<PdfAnalysisOutput> pdfAnalysisOutputs = pdfAnalysisOutputService.queryByPdfId(15);
        List<PdfAnalysisOutput> newPdfAnalysisOutputs = new ArrayList<>();
        for (PdfAnalysisOutput pdfAnalysisOutput : pdfAnalysisOutputs) {
            PdfAnalysisOutput pdf = new PdfAnalysisOutput();
            pdf.setContent(pdfAnalysisOutput.getContent());
            pdf.setPageNo(pdfAnalysisOutput.getPageNo());
            pdf.setDisplayOrder(pdfAnalysisOutput.getDisplayOrder());
            pdf.setTableTitle(pdfAnalysisOutput.getTableTitle());
            pdf.setLayoutType(pdfAnalysisOutput.getLayoutType());
            pdf.setPdfId(16);
            newPdfAnalysisOutputs.add(pdf);
        }
        pdfAnalysisOutputService.saveBatch(newPdfAnalysisOutputs);
    }

    @Autowired
    private TripleToCypherExecutor tripleToCypherExecutor;
    @Test
    void testQueryGraph() {
        CypherSchemaDTO schemaDTO = tripleToCypherExecutor.loadGraphSchema();
        System.out.println(schemaDTO);
    }

    @Test
    void testQueryGraph2() {
        Embedding embedding = aiCallService.embedding("你好");
        System.out.println(embedding.getOutput().length);
    }


    @Autowired
    private IntentionService intentionService;
    @Test
    void testIntentList() {
        List<IntentDTO> intentionDTOs = intentionService.queryByDomainCategoryId("1").stream()
                .filter(intention -> StrUtil.equals("0",intention.getGenerationType())) // 过滤出手动确认的数据
                .map(IntentDTO::new).distinct().toList();
//        List<String> digestList = intentionDTOs.stream().map(IntentDTO::getDigest).toList();
        System.out.println(intentionDTOs.stream().map(i -> "  - " + i.getDigest() + "\n").collect(Collectors.joining()));
    }
    @Autowired
    private DomainMetadataService domainMetadataService;
    @Test
    void testDomainMetadataList() {
        List<DomainMetadataDTO> domainMetadataDTOS = domainMetadataService.listByIntentionIds(List.of("1929814869055692801",
                "1929814915000098818",
                "1929815089067909122",
                "1929815089067909124",
                "1929815178012319747",
                "1929815201403953154",
                "1929815236732575746",
                "1929815236799684610",
                "1929815586533335043",
                "1929815750165716996",
                "1929819219706568708"));
        String domainMetadata = metadataToJsonStr(domainMetadataDTOS);

        System.out.println(domainMetadata);
    }



    /**
     * 将领域元数据转换为json字符串
     * @param domainMetadataDTOS domainMetadataDTOS
     * @return
     */
    private String metadataToJsonStr(List<DomainMetadataDTO> domainMetadataDTOS){
        JSONArray jsa = new JSONArray();
        for (DomainMetadataDTO metadataDTO : domainMetadataDTOS) {
            JSONObject metadataJson = new JSONObject();
            JSONObject source = new JSONObject();
            source.set("type", metadataDTO.getSourceType());
            if (metadataDTO.getSourceAttributes() != null) {
                JSONArray sourceAttributes = new JSONArray();
                for (ERAttributeDTO attribute : metadataDTO.getSourceAttributes()) {
                    sourceAttributes.add(attribute.getAttrName());
                }
                source.set("attributes", sourceAttributes);
            }
            metadataJson.set("source", source);

            JSONObject relation = new JSONObject();
            relation.set("type", metadataDTO.getRelation());
            if (metadataDTO.getRelationAttributes() != null) {
                JSONArray relationAttributes = new JSONArray();
                for (ERAttributeDTO attribute : metadataDTO.getRelationAttributes()) {
                    relationAttributes.add(attribute.getAttrName());
                }
                relation.set("attributes", relationAttributes);
            }

            metadataJson.set("relation", relation);
            JSONObject target = new JSONObject();
            target.set("type", metadataDTO.getTargetType());
            if (metadataDTO.getTargetAttributes() != null) {
                JSONArray targetAttributes = new JSONArray();
                for (ERAttributeDTO attribute : metadataDTO.getTargetAttributes()) {
                    targetAttributes.add(attribute.getAttrName());
                }
                target.set("attributes", targetAttributes);
            }
            metadataJson.set("target", target);
            jsa.add(metadataJson);
        }

        return jsa.toString();
    }
    @Test
    public void domainCategoryListTest() {

        List<DomainMetadataDTO> domainMetadataDTOS = domainMetadataService.listByIntentionIds(List.of("1928020787140612097"), "1");
        System.out.println(domainMetadataDTOS);
    }


    @Autowired
    private TextVectorService textVectorService;
    @Test
    public void textVectorTest() {
        String texts = """
                公司
                集团
                营收
                金额
                时间
                """;
        String[] split = texts.split("\n");
        List<String> list = Arrays.stream(split).toList();
        for (String text : list) {
            TextVector textVector = new TextVector();
            textVector.setContent(text.trim());
            textVector.setCategoryId("分词");
            float[] output = aiCallService.embedding(textVector.getContent()).getOutput();
            textVector.setEmbedding(output);
            textVectorService.save(textVector);
        }

    }
    @Test
    public void textVectorTest2() {
        // 龙源电力集团近三年营收情况是多少
        String queryText = "龙源电力集团近三年营收情况是多少";
        float[] output = aiCallService.embedding(queryText).getOutput();
        List<TextVectorDTO> similarByCosine = textVectorService.findSimilarByCosine(output, 0.1f, 5);
        similarByCosine = similarByCosine.stream().sorted(Comparator.comparingDouble(TextVectorDTO::getSimilarityScore).reversed()).collect(Collectors.toList());
        log.info("<<<===========================>>>" );
        for (TextVectorDTO vectorDTO : similarByCosine) {
            String content = vectorDTO.getContent();
            double similarityScore = vectorDTO.getSimilarityScore();
            String categoryId = vectorDTO.getCategoryId();
            System.out.printf("%s\t%s\t%s\t%s%n",queryText,  categoryId , NumberUtil.decimalFormat("0.0000",similarityScore),content);
        }
    }

    @Autowired
    private Retriever retriever;

    @Autowired
    private TextToSegmentService textToSegmentService;
    @Test
    public void textVectorTest3() {

      // tripleToCypherExecutor.refreshSchemaSegmentVector();


        TimeInterval timer = new TimeInterval();
        textToSegmentService.addDict("龙源电力集团","企业",1000);
        List<Map<String, Object>> retrieval = retriever.retrieval("龙源电力集团近三年营收情况是多少");
        System.out.println(JSONUtil.toJsonStr(retrieval));
        log.info("<<<===========================>>> 耗时: {} 毫秒", timer.intervalMs());
    }
}
